#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from transaction.interfaces import ITransactionManager
from onf_website.features.header.interfaces import IDefaultHeaderRendererSettings, IDefaultHeaderMenu, \
    IDefaultHeaderMenusContainer
from pyams_form.interfaces.form import IInnerSubForm
from pyams_portal.interfaces import MANAGE_TEMPLATE_PERMISSION
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.features.menu.zmi import LinksTable, MenuLinksView, IMenusView, MenusTable, IMenuLinksView, \
    MenusView
from pyams_content.features.header.zmi import HeaderSettingsRendererSettingsEditForm
from pyams_form.group import NamedWidgetsGroup
from pyams_skin.event import get_json_widget_refresh_event
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import InnerAdminEditForm
from z3c.form import field
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget

from onf_website import _


@adapter_config(name='default-header-properties',
                context=(IDefaultHeaderRendererSettings, IPyAMSLayer, HeaderSettingsRendererSettingsEditForm),
                provides=IInnerSubForm)
class DefaultHeaderPropertiesEditForm(InnerAdminEditForm):
    """Default header properties edit form"""

    legend = None

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    weight = 1

    @property
    def fields(self):
        fields = field.Fields(IDefaultHeaderRendererSettings).omit('use_context_illustration')
        if not self.context.can_use_context_illustration:
            fields = fields.omit('no_use_context_illustration')
        else:
            fields['no_use_context_illustration'].widgetFactory = SingleCheckBoxFieldWidget
        fields['search_menu'].widgetFactory = SingleCheckBoxFieldWidget
        return fields

    def updateGroups(self):
        if self.context.can_use_context_illustration:
            group = NamedWidgetsGroup(self, 'background', self.widgets,
                                      ('no_use_context_illustration', 'background'),
                                      legend=_("Don't use context illustration as background"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_mode='hide',
                                      checkbox_field=IDefaultHeaderRendererSettings['no_use_context_illustration'])
        else:
            group = NamedWidgetsGroup(self, 'background', self.widgets,
                                      ('background', ))
        self.add_group(group)
        self.add_group(NamedWidgetsGroup(self, 'properties', self.widgets,
                                         ('heading', 'logo', 'scroll_logo', 'social_links_header', 'accessibility_menu')))
        self.add_group(NamedWidgetsGroup(self, 'search', self.widgets,
                                         ('search_menu', 'search_target', 'experiential_engine'),
                                         legend=_("Show search menu"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_mode='hide',
                                         checkbox_field=IDefaultHeaderRendererSettings['search_menu']))
        super(DefaultHeaderPropertiesEditForm, self).updateGroups()

    def get_ajax_output(self, changes):
        output = super(DefaultHeaderPropertiesEditForm, self).get_ajax_output(changes)
        updated = changes.get(IDefaultHeaderRendererSettings, ())
        if updated:
            ITransactionManager(self.context).commit()
            if 'background' in updated:
                output.setdefault('events', []).append(
                    get_json_widget_refresh_event(self.context, self.request,
                                                  DefaultHeaderPropertiesEditForm, 'background'))
            if 'logo' in updated:
                output.setdefault('events', []).append(
                    get_json_widget_refresh_event(self.context, self.request,
                                                  DefaultHeaderPropertiesEditForm, 'logo'))
            if 'scroll_logo' in updated:
                output.setdefault('events', []).append(
                    get_json_widget_refresh_event(self.context, self.request,
                                                  DefaultHeaderPropertiesEditForm, 'scroll_logo'))
        return output


#
# Header top links table view
#

class TopLinksAssociationsTable(LinksTable):
    """Default header top links associations table"""

    associations_name = 'top'


@adapter_config(name='default-header-top-links',
                context=(IDefaultHeaderRendererSettings, IPyAMSLayer, HeaderSettingsRendererSettingsEditForm),
                provides=IInnerSubForm)
@adapter_config(name='++ass++top', context=(IDefaultHeaderMenu, IPyAMSLayer), provides=IMenuLinksView)
class DefaultHeaderTopLinksView(MenuLinksView):
    """Default header top links view"""

    title = _("Top row links")

    table_class = TopLinksAssociationsTable
    weight = 10


#
# Header menus table view
#

@adapter_config(name='default-header-menus',
                context=(IDefaultHeaderRendererSettings, IPyAMSLayer, HeaderSettingsRendererSettingsEditForm),
                provides=IInnerSubForm)
@adapter_config(name='++ass++menus', context=(IDefaultHeaderMenusContainer, IPyAMSLayer), provides=IMenusView)
class DefaultHeaderMenusView(MenusView):
    """Default header middle groups view"""

    title = _("Header menus")

    table_class = MenusTable
    weight = 11


#
# Header social links table view
#

class SocialLinksAssociationsTable(LinksTable):
    """Default header social links associations table"""

    associations_name = 'social'


@adapter_config(name='default-header-social-links',
                context=(IDefaultHeaderRendererSettings, IPyAMSLayer, HeaderSettingsRendererSettingsEditForm),
                provides=IInnerSubForm)
@adapter_config(name='++ass++social', context=(IDefaultHeaderMenu, IPyAMSLayer), provides=IMenuLinksView)
class DefaultHeaderSocialLinksView(MenuLinksView):
    """Default header social links view"""

    title = _("Social networks links")

    table_class = SocialLinksAssociationsTable
    weight = 15
