#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Interface, implementer
from zope.location import Location
from zope.schema.fieldproperty import FieldProperty

from onf_website.features.menu.skin.interfaces import IONFExternalNavigationPortletRendererSettings, \
    IONFNavigationPortletRendererSettings, IONFCatalogNavigationPortletRendererSettings, \
    IONFNavigationPortletRendererSettingsWithSubtitle, IONFCarouselNavigationPortletRendererSettings
from onf_website.skin.public import aside_sticky, owl_carousel_init, owl_navigation
from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.component.links import InternalReferenceMixin, IInternalLink
from pyams_content.features.menu.portlet.navigation.interfaces.simple import ISimpleNavigationPortletSettings
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.portlet import PortletRenderer
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config

from onf_website import _


#
# Generic navigation portlet settings
#

@implementer(IONFNavigationPortletRendererSettings)
@factory_config(provided=IONFNavigationPortletRendererSettings)
class ONFNavigationPortletRendererSettings(Persistent, Location):
    """ONF navigation portlet renderer settings"""

    header = FieldProperty(IONFNavigationPortletRendererSettings['header'])


@implementer(IONFNavigationPortletRendererSettingsWithSubtitle)
@factory_config(provided=IONFNavigationPortletRendererSettingsWithSubtitle)
class ONFNavigationPortletRendererSettingsWithSubtitle(ONFNavigationPortletRendererSettings):
    """ONF navigation portlet renderer settings with subtitle"""

    subtitle = FieldProperty(IONFNavigationPortletRendererSettingsWithSubtitle['subtitle'])


#
# Base simple navigation portlet renderer
#

class ONFBaseSimpleNavigationPortletRenderer(PortletRenderer):
    """ONF base simple navigation portlet renderer"""

    settings_interface = IONFNavigationPortletRendererSettings

    @staticmethod
    def get_link_info(link):
        return IAssociationInfo(link)


class ONFBaseSimpleNavigationPortletRendererWithSubtitle(ONFBaseSimpleNavigationPortletRenderer):
    """ONF base simple navigation portlet renderer including subtitle"""

    settings_interface = IONFNavigationPortletRendererSettingsWithSubtitle


#
# Horizontal tabs navigation portlet renderer
#

@adapter_config(name='ONF::navigation-tabs',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-tabs.pt', layer=IPyAMSLayer)
class ONFNavigationTabsPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF horizontal tabs navigation portlet renderer"""

    label = _("ONF: Horizontal navigation tabs")
    weight = 10


#
# Horizontal banner navigation portlet renderer
#

@adapter_config(name='ONF::navigation-banner',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-banner.pt', layer=IPyAMSLayer)
class ONFNavigationBannerPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF horizontal banner navigation portlet renderer"""

    label = _("ONF: Horizontal navigation banner")
    weight = 11

    settings_interface = None

    def get_tags_viewlet(self, link):
        if not IInternalLink.providedBy(link):
            return ''
        target = link.get_target()
        if target is not None:
            registry = self.request.registry
            viewlet = registry.queryMultiAdapter((target, self.request, self), IContentProvider, name='pyams.tags')
            if viewlet is not None:
                viewlet.update()
                return viewlet.render()
        return ''


#
# Horizontal navigation with background illustrations portlet renderer
#

@adapter_config(name='ONF::navigation-illustration-shortcuts',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-shortcuts.pt', layer=IPyAMSLayer)
class ONFShortcutNavigationPortletRenderer(ONFBaseSimpleNavigationPortletRendererWithSubtitle):
    """ONF simple navigation horizontal portlet renderer with tabs"""

    label = _("ONF: Horizontal links list with background illustrations")
    weight = 12


#
# Vertical illustrations portlet renderer
#

@adapter_config(name='ONF::navigation-illustration-vertical',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-illustration-vertical.pt', layer=IPyAMSLayer)
class ONFNavigationIllustrationVerticalPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF vertical illustrations navigation portlet renderer"""

    label = _("ONF: Vertical illustrations navigation")
    weight = 13

    resources = (owl_carousel_init, owl_navigation)


#
# External navigation portlet renderer
#

@implementer(IONFExternalNavigationPortletRendererSettings)
@factory_config(provided=IONFExternalNavigationPortletRendererSettings)
class ONFExternalNavigationPortletRendererSettings(Persistent, Location):
    """ONF external navigation portlet renderer settings"""

    button_label = FieldProperty(IONFExternalNavigationPortletRendererSettings['button_label'])


@adapter_config(name='ONF::navigation-external-shortcuts',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-external-shortcuts.pt', layer=IPyAMSLayer)
class ONFExternalShortcutNavigationPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF External navigation portlet renderer"""

    label = _("ONF: External navigation links with illustrations")
    weight = 14

    settings_interface = IONFExternalNavigationPortletRendererSettings


#
# Useful links vertical navigation portlet renderer
#

@adapter_config(name='ONF::navigation-vertical-useful-links',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-vertical-useful-links.pt', layer=IPyAMSLayer)
class ONFNavigationVerticalUsefulLinksPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF Useful links vertical navigation portlet renderer"""

    label = _("ONF: Useful links vertical list")
    weight = 15

    settings_interface = None

    resources = (aside_sticky,)


#
# Info links vertical navigation portlet renderer
#

@adapter_config(name='ONF::navigation-vertical-info-links',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-vertical-info-links.pt', layer=IPyAMSLayer)
class ONFNavigationVerticalInfoLinksPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF Info links vertical navigation portlet renderer"""

    label = _("ONF: Info links button vertical list")
    weight = 16

    resources = (aside_sticky,)
    settings_interface = None


#
# Catalog navigation portlet renderer
#

@implementer(IONFCatalogNavigationPortletRendererSettings)
@factory_config(provided=IONFCatalogNavigationPortletRendererSettings)
class ONFCatalogNavigationPortletRendererSettings(Persistent, Location, InternalReferenceMixin):
    """ONF catalog navigation portlet renderer settings"""

    teaser = FieldProperty(IONFCatalogNavigationPortletRendererSettings['teaser'])
    button_label = FieldProperty(IONFCatalogNavigationPortletRendererSettings['button_label'])
    reference = FieldProperty(IONFCatalogNavigationPortletRendererSettings['reference'])


@adapter_config(name='ONF::navigation-catalog',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-catalog.pt', layer=IPyAMSLayer)
class ONFCatalogNavigationPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF catalog navigation portlet renderer"""

    label = _("ONF: Catalog horizontal renderer")
    weight = 17

    settings_interface = IONFCatalogNavigationPortletRendererSettings


#
# Single navigation portlet renderer
#

@adapter_config(name='ONF::navigation-single-link',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-single-link.pt', layer=IPyAMSLayer)
class ONFSingleLinkNavigationPortletRenderer(ONFBaseSimpleNavigationPortletRenderer):
    """ONF Single link navigation portlet renderer"""

    label = _("ONF: Single link navigation banner")
    weight = 18

    settings_interface = None


#
# Carousel navigation portlet renderer
#

@implementer(IONFCarouselNavigationPortletRendererSettings)
@factory_config(provided=IONFCarouselNavigationPortletRendererSettings)
class ONFCarouselNavigationPortletRendererSettings(Persistent, Location):
    """ONF carousel navigation portlet renderer settings"""

    teaser = FieldProperty(IONFCarouselNavigationPortletRendererSettings['teaser'])


@adapter_config(name='ONF::navigation-carousel',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-carousel.pt', layer=IPyAMSLayer)
class ONFCarouselNavigationPortletRenderer(ONFNavigationBannerPortletRenderer):
    """ONF Carousel navigation with teaser portlet renderer"""

    label = _("ONF: Carousel horizontal navigation renderer ")
    weight = 19

    resources = (owl_carousel_init,)
    settings_interface = IONFCarouselNavigationPortletRendererSettings


#
# Grid navigation portlet renderer
#

@adapter_config(name='ONF::navigation-grid',
                context=(IPortalContext, IPyAMSLayer, Interface, ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/navigation-grid.pt', layer=IPyAMSLayer)
class ONFGridNavigationPortletRenderer(ONFNavigationBannerPortletRenderer):
    """ONF Grid navigation portlet renderer"""

    label = _("ONF: Grid navigation renderer ")
    weight = 20

    settings_interface = IONFNavigationPortletRendererSettingsWithSubtitle
